//-----------------------------------------------------------------------------
//
//	picdasm PIC microcontroller disassember
//
//-----------------------------------------------------------------------------
//
//	Cosmodog, Ltd.
//	415 West Superior Street
//	Chicago, IL   60610-3429
//	http://www.cosmodog.com
//
//-----------------------------------------------------------------------------
//
//	Copyright (C) 1999 Cosmodog, Ltd.
//
//	This program is free software; you can redistribute it and/or
//	modify it under the terms of the GNU General Public License
//	as published by the Free Software Foundation; either version 2
//	of the License, or (at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//-----------------------------------------------------------------------------
//
//	version 0.1c:
//		initial release
//
//	version 0.1d:
//		vastly improve and generalize processor definitions
//		fix several small errors in processor definitions
//		properly select a processor to disassemble
//
//-----------------------------------------------------------------------------


#include "includes.h"


static const char versionString[] =	"0.1d";	// this program's version number


extern int errno;

char *programName;
char *procName;
char *infileName;
char *outfileName;

FILE *inFile;
FILE *outFile;



// dump a decorative header
//
static void DoHeader(FILE *outFile, const PIC_DEF *procDef)
{
	fprintf(outFile,";------------------------------------------\n");
	fprintf(outFile,";\n");
	fprintf(outFile,"; %s version %s\n",programName,versionString);
	if(inFile != stdin)
	{
		fprintf(outFile,"; disassembly of %s\n", infileName);		// report file name only if input came from a file
	}
	fprintf(outFile,"; processor type: %s\n",procDef->name);
	fprintf(outFile,";\n");
	fprintf(outFile,";------------------------------------------\n");
	fprintf(outFile,"\n");
}


// disassemble the input, send it to the output
//
static void DoDisassemble(const PIC_DEF *procDef)
{
	UINT32 byteAddr;
	UINT8 data;
	BOOLEAN done = FALSE;
	UINT16 instruction;
	UINT16 address;



	DoHeader(outFile, procDef);											// dump a decorative header
	InitParse();														// get ready to parse the input file
	while(!done)
	{
		done = !GetNextByte(inFile, &byteAddr, &data);					// read low byte
		if(!done)
		{
			instruction = data;
			address = byteAddr;											// use the address of the low byte
			done = !GetNextByte(inFile, &byteAddr, &data);				// read high byte
			if(!done)
			{
				instruction+=(data<<8);
				DisassembleInstruction(outFile, instruction, address, procDef);			// disassemble and display the instruction
			}
		}
	}
}

static void Usage()
// tell the user how to use this program
{
	fprintf(stderr,"%s: version %s\n",programName,versionString);
	fprintf(stderr,"Usage: %s devtype [infile [outfile]]\n",programName);
	fprintf(stderr,"where:\n");
	fprintf(stderr,"  devtype is the pic device to be used (PIC12C508, PIC16C505, etc.)\n");
	fprintf(stderr,"  infile is an optional file to disassemble\n");
	fprintf(stderr,"  outfile is an optional output file\n");
	fprintf(stderr,"if no infile is specified input is read from stdin\n");
	fprintf(stderr,"if no outfile is specified output is sent to stdout\n");
	fprintf(stderr,"infile may be Intel hex or Motorola S format\n");
	ShowDevices();
}

int main(int argc,char *argv[])
{
	BOOLEAN fail = false;
	programName=argv[0];
	const PIC_DEF *procDef;
	
	InitReportErrors();
	
	if(argc>1)
	{
		procName = argv[1];
		procDef = InitDisassemble(procName);					// arg1 is processor type; try to initialize it
		if(procDef)												
		{
			inFile = stdin;										// assume no files specified, use stdin/stdout instead
			outFile = stdout;
			if(argc>2)
			{
				infileName = argv[2];
				if( (inFile=OpenReadFile(infileName,0x10000)) )	// specify an input file
				{
					if(argc>3)
					{
						outfileName = argv[3];
						if( !(outFile=OpenWriteFile(outfileName,0x10000)) )
						{
							ReportError("unable to create output file: '%s'\n",outfileName);
							fail = TRUE;
						}
					}
				}
				else
				{
					ReportError("unable to open input file: '%s'\n",infileName);
					fail = TRUE;
				}
								
			}
		}
		else
		{
			ReportError("unable to find processor: '%s'\n",procName);
			fail = TRUE;
		}
		
		if(!fail)
		{
			DoDisassemble(procDef);				// disassemble
			if(inFile != stdin)					// if there was an input file,
			{
				CloseFile(inFile);				// close it
			}
			if(outFile != stdout)				// if there was an output file,
			{
				CloseFile(outFile);				// close it
			}
		}
	}
	else
	{
		Usage();
	}
	return(0);
}
